import threading
from logger import Logger
import paho.mqtt.client as mqtt
import json
import datetime

_parsers = {}


def on_message(client, userdata, msg):
    try:
        j = {'payload':msg.payload, 'topic':msg.topic, 'timestamp':datetime.datetime.now()}

        Logger.debug("MqttReceiver receives: %s" % j)

        global _parsers
        _parsers[msg.topic].execute(j)
    except ValueError, e:
        Logger.log("Exception %s in MqttReceiver, on_message" % (str(e)))


class MqttReceiver(threading.Thread):
    singleton = None
    
    @classmethod
    def create(cls, broker):
        if cls.singleton is None:
            cls.singleton = MqttReceiver(broker)
        return cls.singleton
            
    def __init__(self, broker):
        threading.Thread.__init__(self)
        self.client = mqtt.Client()
        self.client.on_message = on_message
        self.client.connect(broker, 1883, 60)
        self.setDaemon(True)

    def registerParser(self, parser):
        Logger.log("registering %s for topic %s" % (parser.__class__.__name__, parser.getTopic()))
        self.client.subscribe(parser.getTopic())
        global _parsers
        _parsers[parser.getTopic()] = parser

    def run(self):
        self.client.loop_forever()
